@@ -62,7 +62,6 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             let mut p = F32;
             r8!(asm!("
-                        ldr w19, [{0}]
                         ldr w20, [{0}]
                         ldr w21, [{0}]
                         ldr w22, [{0}]
@@ -70,10 +69,11 @@ pub fn ld_64F32(c: &mut Criterion) {
                         ldr w24, [{0}]
                         ldr w25, [{0}]
                         ldr w26, [{0}]
+                        ldr w27, [{0}]
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             ));
         })
     });
@@ -81,7 +81,6 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             let mut p = F32;
             r8!(asm!("
-                        ldr x19, [{0}]
                         ldr x20, [{0}]
                         ldr x21, [{0}]
                         ldr x22, [{0}]
@@ -89,10 +88,11 @@ pub fn ld_64F32(c: &mut Criterion) {
                         ldr x24, [{0}]
                         ldr x25, [{0}]
                         ldr x26, [{0}]
+                        ldr x27, [{0}]
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             ));
         })
     });
@@ -211,19 +211,16 @@ pub fn ld_64F32(c: &mut Criterion) {
     group.bench_function("ins_32b", |b| {
         b.iter(|| unsafe {
             r8!(asm!("
-                ins v8.s[0], w19
-                ins v9.s[0], w19
-                ins v10.s[0], w19
-                ins v11.s[0], w19
-                ins v12.s[0], w19
-                ins v13.s[0], w19
-                ins v14.s[0], w19
-                ins v15.s[0], w19
+                ins v8.s[0], w20
+                ins v9.s[0], w20
+                ins v10.s[0], w20
+                ins v11.s[0], w20
+                ins v12.s[0], w20
+                ins v13.s[0], w20
+                ins v14.s[0], w20
+                ins v15.s[0], w20
                 ",
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
-            out("v0") _, out("v1") _, out("v2") _, out("v3") _,
-            out("v4") _, out("v5") _, out("v6") _, out("v7") _,
+            out("x20") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
             out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
@@ -234,42 +231,41 @@ pub fn ld_64F32(c: &mut Criterion) {
             r4!(asm!("
         ins         v0.s[0], w9
         ins         v1.s[0], w13
-            ins         v4.s[0], w19
-            ins         v5.s[0], w23
+            ins         v4.s[0], w20
+            ins         v5.s[0], w24
         ins         v0.s[1], w10
         ins         v1.s[1], w14
-            ins         v4.s[1], w20
-            ins         v5.s[1], w24
+            ins         v4.s[1], w21
+            ins         v5.s[1], w25
         ins         v0.s[2], w11
         ins         v1.s[2], w15
-            ins         v4.s[2], w21
-            ins         v5.s[2], w25
+            ins         v4.s[2], w22
+            ins         v5.s[2], w26
         ins         v0.s[3], w12
         ins         v1.s[3], w4
-            ins         v4.s[3], w22
-            ins         v5.s[3], w26
+            ins         v4.s[3], w23
+            ins         v5.s[3], w27
                 ",
             out("v0") _, out("v1") _,
             out("v4") _, out("v5") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             ));
         })
     });
     group.bench_function("ins_64b", |b| {
         b.iter(|| unsafe {
             r8!(asm!("
-                ins v8.d[0], x19
-                ins v9.d[0], x19
-                ins v10.d[0], x19
-                ins v11.d[0], x19
-                ins v12.d[0], x19
-                ins v13.d[0], x19
-                ins v14.d[0], x19
-                ins v15.d[0], x19
+                ins v8.d[0], x20
+                ins v9.d[0], x20
+                ins v10.d[0], x20
+                ins v11.d[0], x20
+                ins v12.d[0], x20
+                ins v13.d[0], x20
+                ins v14.d[0], x20
+                ins v15.d[0], x20
                 ",
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
-            out("v0") _, out("v1") _, out("v2") _, out("v3") _,
-            out("v4") _, out("v5") _, out("v6") _, out("v7") _,
+            out("x20") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
             out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
@@ -279,30 +275,28 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             let mut p = F32;
             r8!(asm!("
-                        ldr w19, [{0}]
-                        fmla v0.4s, v0.4s, v0.4s
                         ldr w20, [{0}]
-                        fmla v1.4s, v1.4s, v1.4s
+                        fmla v0.4s, v0.4s, v0.4s
                         ldr w21, [{0}]
-                        fmla v2.4s, v2.4s, v2.4s
+                        fmla v1.4s, v1.4s, v1.4s
                         ldr w22, [{0}]
-                        fmla v3.4s, v3.4s, v3.4s
+                        fmla v2.4s, v2.4s, v2.4s
                         ldr w23, [{0}]
-                        fmla v4.4s, v4.4s, v4.4s
+                        fmla v3.4s, v3.4s, v3.4s
                         ldr w24, [{0}]
-                        fmla v5.4s, v5.4s, v5.4s
+                        fmla v4.4s, v4.4s, v4.4s
                         ldr w25, [{0}]
-                        fmla v6.4s, v6.4s, v6.4s
+                        fmla v5.4s, v5.4s, v5.4s
                         ldr w26, [{0}]
+                        fmla v6.4s, v6.4s, v6.4s
+                        ldr w27, [{0}]
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
-            out("v8") _, out("v9") _, out("v10") _, out("v11") _,
-            out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
         })
     });
@@ -310,30 +304,28 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             let mut p = F32;
             r8!(asm!("
-                        ldr w19, [{0}], #4
-                        fmla v0.4s, v0.4s, v0.4s
                         ldr w20, [{0}], #4
-                        fmla v1.4s, v1.4s, v1.4s
+                        fmla v0.4s, v0.4s, v0.4s
                         ldr w21, [{0}], #4
-                        fmla v2.4s, v2.4s, v2.4s
+                        fmla v1.4s, v1.4s, v1.4s
                         ldr w22, [{0}], #4
-                        fmla v3.4s, v3.4s, v3.4s
+                        fmla v2.4s, v2.4s, v2.4s
                         ldr w23, [{0}], #4
-                        fmla v4.4s, v4.4s, v4.4s
+                        fmla v3.4s, v3.4s, v3.4s
                         ldr w24, [{0}], #4
-                        fmla v5.4s, v5.4s, v5.4s
+                        fmla v4.4s, v4.4s, v4.4s
                         ldr w25, [{0}], #4
-                        fmla v6.4s, v6.4s, v6.4s
+                        fmla v5.4s, v5.4s, v5.4s
                         ldr w26, [{0}], #4
+                        fmla v6.4s, v6.4s, v6.4s
+                        ldr w27, [{0}], #4
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
-            out("v8") _, out("v9") _, out("v10") _, out("v11") _,
-            out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
         })
     });
@@ -342,30 +334,28 @@ pub fn ld_64F32(c: &mut Criterion) {
             let mut p = F32;
             let mut q = F32;
             r8!(asm!("
-                        ldr w19, [{0}], #4
+                        ldr w20, [{0}], #4
                         fmla v0.4s, v0.4s, v0.4s
-                        ldr w20, [{1}], #4
+                        ldr w21, [{1}], #4
                         fmla v1.4s, v1.4s, v1.4s
-                        ldr w21, [{0}], #4
+                        ldr w22, [{0}], #4
                         fmla v2.4s, v2.4s, v2.4s
-                        ldr w22, [{1}], #4
+                        ldr w23, [{1}], #4
                         fmla v3.4s, v3.4s, v3.4s
-                        ldr w23, [{0}], #4
+                        ldr w24, [{0}], #4
                         fmla v4.4s, v4.4s, v4.4s
-                        ldr w24, [{1}], #4
+                        ldr w25, [{1}], #4
                         fmla v5.4s, v5.4s, v5.4s
-                        ldr w25, [{0}], #4
+                        ldr w26, [{0}], #4
                         fmla v6.4s, v6.4s, v6.4s
-                        ldr w26, [{1}], #4
+                        ldr w27, [{1}], #4
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p, inout(reg) q,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
-            out("v8") _, out("v9") _, out("v10") _, out("v11") _,
-            out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
         })
     });
@@ -373,30 +363,28 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             let mut p = F32;
             r8!(asm!("
-                        ldr w19, [{0}]
+                        ldr w20, [{0}]
                         fmla v0.4s, v0.4s, v0.4s
-                        ldr w20, [{0}, #4]
+                        ldr w21, [{0}, #4]
                         fmla v1.4s, v1.4s, v1.4s
-                        ldr w21, [{0}, #8]
+                        ldr w22, [{0}, #8]
                         fmla v2.4s, v2.4s, v2.4s
-                        ldr w22, [{0}, #12]
+                        ldr w23, [{0}, #12]
                         fmla v3.4s, v3.4s, v3.4s
-                        ldr w23, [{0}, #16]
+                        ldr w24, [{0}, #16]
                         fmla v4.4s, v4.4s, v4.4s
-                        ldr w24, [{0}, #20]
+                        ldr w25, [{0}, #20]
                         fmla v5.4s, v5.4s, v5.4s
-                        ldr w25, [{0}, #24]
+                        ldr w26, [{0}, #24]
                         fmla v6.4s, v6.4s, v6.4s
-                        ldr w26, [{0}, #28]
+                        ldr w27, [{0}, #28]
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
-            out("v8") _, out("v9") _, out("v10") _, out("v11") _,
-            out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
         })
     });
@@ -405,29 +393,27 @@ pub fn ld_64F32(c: &mut Criterion) {
             let mut p = F32;
             r8!(asm!("
                 fmla v0.4s, v0.4s, v0.4s
-                ldr x19, [{0}]
-                fmla v1.4s, v1.4s, v1.4s
                 ldr x20, [{0}]
-                fmla v2.4s, v2.4s, v2.4s
+                fmla v1.4s, v1.4s, v1.4s
                 ldr x21, [{0}]
-                fmla v3.4s, v3.4s, v3.4s
+                fmla v2.4s, v2.4s, v2.4s
                 ldr x22, [{0}]
-                fmla v4.4s, v4.4s, v4.4s
+                fmla v3.4s, v3.4s, v3.4s
                 ldr x23, [{0}]
-                fmla v5.4s, v5.4s, v5.4s
+                fmla v4.4s, v4.4s, v4.4s
                 ldr x24, [{0}]
-                fmla v6.4s, v6.4s, v6.4s
+                fmla v5.4s, v5.4s, v5.4s
                 ldr x25, [{0}]
-                fmla v7.4s, v7.4s, v7.4s
+                fmla v6.4s, v6.4s, v6.4s
                 ldr x26, [{0}]
+                fmla v7.4s, v7.4s, v7.4s
+                ldr x27, [{0}]
                 ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+            out("x24") _, out("x25") _, out("x26") _, out("x27") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
-            out("v8") _, out("v9") _, out("v10") _, out("v11") _,
-            out("v12") _, out("v13") _, out("v14") _, out("v15") _,
             ));
         })
     });
@@ -453,8 +439,6 @@ pub fn ld_64F32(c: &mut Criterion) {
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -484,12 +468,12 @@ pub fn ld_64F32(c: &mut Criterion) {
                         fmla v7.4s, v7.4s, v7.4s
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
             out("v12") _, out("v13") _, out("v14") _, out("v15") _,
+            out("v16") _, out("v17") _, out("v18") _, out("v19") _,
+            out("v20") _, out("v21") _, out("v22") _, out("v23") _,
             ));
         })
     });
@@ -515,8 +499,6 @@ pub fn ld_64F32(c: &mut Criterion) {
                 ld1 {{ v16.4s }}, [{0}]
                       ",
             inout(reg) p,
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -528,28 +510,27 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             r8!(asm!("
                 fmla v0.4s, v0.4s, v0.4s
-                ins v8.s[0], w19
+                ins v8.s[0], w20
                 fmla v1.4s, v1.4s, v1.4s
-                ins v9.s[0], w19
+                ins v9.s[0], w20
                 fmla v2.4s, v2.4s, v2.4s
-                ins v10.s[0], w19
+                ins v10.s[0], w20
                 fmla v3.4s, v3.4s, v3.4s
-                ins v11.s[0], w19
+                ins v11.s[0], w20
                 fmla v4.4s, v4.4s, v4.4s
-                ins v12.s[0], w19
+                ins v12.s[0], w20
                 fmla v5.4s, v5.4s, v5.4s
-                ins v13.s[0], w19
+                ins v13.s[0], w20
                 fmla v6.4s, v6.4s, v6.4s
-                ins v14.s[0], w19
+                ins v14.s[0], w20
                 fmla v7.4s, v7.4s, v7.4s
-                ins v15.s[0], w19
+                ins v15.s[0], w20
                 ",
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
             out("v12") _, out("v13") _, out("v14") _, out("v15") _,
+            out("x20") _,
             ));
         })
     });
@@ -557,24 +538,23 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             r8!(asm!("
                 fmla v0.4s, v0.4s, v0.4s
-                ins v8.d[0], x19
+                ins v8.d[0], x20
                 fmla v1.4s, v1.4s, v1.4s
-                ins v9.d[0], x19
+                ins v9.d[0], x20
                 fmla v2.4s, v2.4s, v2.4s
-                ins v10.d[0], x19
+                ins v10.d[0], x20
                 fmla v3.4s, v3.4s, v3.4s
-                ins v11.d[0], x19
+                ins v11.d[0], x20
                 fmla v4.4s, v4.4s, v4.4s
-                ins v12.d[0], x19
+                ins v12.d[0], x20
                 fmla v5.4s, v5.4s, v5.4s
-                ins v13.d[0], x19
+                ins v13.d[0], x20
                 fmla v6.4s, v6.4s, v6.4s
-                ins v14.d[0], x19
+                ins v14.d[0], x20
                 fmla v7.4s, v7.4s, v7.4s
-                ins v15.d[0], x19
+                ins v15.d[0], x20
                 ",
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -586,24 +566,23 @@ pub fn ld_64F32(c: &mut Criterion) {
         b.iter(|| unsafe {
             r8!(asm!("
                 fmla v0.4s, v0.4s, v0.4s
-                ins v9.d[0], x19
+                ins v9.d[0], x20
                 fmla v1.4s, v1.4s, v1.4s
-                ins v10.d[0], x19
+                ins v10.d[0], x20
                 fmla v2.4s, v2.4s, v2.4s
-                ins v11.d[0], x19
+                ins v11.d[0], x20
                 fmla v3.4s, v6.4s, v3.4s
-                ins v12.d[0], x19
+                ins v12.d[0], x20
                 fmla v4.4s, v4.4s, v4.4s
-                ins v13.d[0], x19
+                ins v13.d[0], x20
                 fmla v5.4s, v5.4s, v5.4s
-                ins v14.d[0], x19
+                ins v14.d[0], x20
                 fmla v6.4s, v6.4s, v6.4s
-                ins v15.d[0], x19
+                ins v15.d[0], x20
                 fmla v7.4s, v7.4s, v7.4s
-                ins v8.d[0], x19
+                ins v8.d[0], x20
                 ",
-            out("x19") _, out("x20") _, out("x21") _, out("x22") _,
-            out("x23") _, out("x24") _, out("x25") _, out("x26") _,
+            out("x20") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -616,23 +595,24 @@ pub fn ld_64F32(c: &mut Criterion) {
             let mut p = F32;
             r8!(asm!("
                 ldr s0, [{0}]
-                ins v8.d[0], x19
+                ins v8.d[0], x20
                 ldr s1, [{0}]
-                ins v9.d[0], x19
+                ins v9.d[0], x20
                 ldr s2, [{0}]
-                ins v10.d[0], x19
+                ins v10.d[0], x20
                 ldr s3, [{0}]
-                ins v11.d[0], x19
+                ins v11.d[0], x20
                 ldr s4, [{0}]
-                ins v12.d[0], x19
+                ins v12.d[0], x20
                 ldr s5, [{0}]
-                ins v13.d[0], x19
+                ins v13.d[0], x20
                 ldr s6, [{0}]
-                ins v14.d[0], x19
+                ins v14.d[0], x20
                 ldr s7, [{0}]
-                ins v15.d[0], x19
+                ins v15.d[0], x20
                 ",
             inout(reg) p,
+            out("x20") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -645,23 +625,24 @@ pub fn ld_64F32(c: &mut Criterion) {
             let mut p = F32;
             r8!(asm!("
                 ldr s0, [{0}]
-                ins v9.d[0], x19
+                ins v9.d[0], x20
                 ldr s1, [{0}]
-                ins v10.d[0], x19
+                ins v10.d[0], x20
                 ldr s2, [{0}]
-                ins v11.d[0], x19
+                ins v11.d[0], x20
                 ldr s3, [{0}]
-                ins v12.d[0], x19
+                ins v12.d[0], x20
                 ldr s4, [{0}]
-                ins v13.d[0], x19
+                ins v13.d[0], x20
                 ldr s5, [{0}]
-                ins v14.d[0], x19
+                ins v14.d[0], x20
                 ldr s6, [{0}]
-                ins v15.d[0], x19
+                ins v15.d[0], x20
                 ldr s7, [{0}]
-                ins v8.d[0], x19
+                ins v8.d[0], x20
                 ",
             inout(reg) p,
+            out("x20") _,
             out("v0") _, out("v1") _, out("v2") _, out("v3") _,
             out("v4") _, out("v5") _, out("v6") _, out("v7") _,
             out("v8") _, out("v9") _, out("v10") _, out("v11") _,
@@ -686,7 +667,7 @@ fn packed_packed_8x8_loop1(c: &mut Criterion) {
                     out("x4") _, out("x5") _, out("x6") _, out("x7") _,
                     out("x8") _, out("x9") _, out("x10") _, out("x11") _,
                     out("x12") _, out("x13") _, out("x14") _, out("x15") _,
-                    out("x19") _, out("x20") _, out("x21") _, out("x22") _,
+                    out("x20") _, out("x21") _, out("x22") _,
                     out("x23") _, out("x24") _, out("x25") _, out("x26") _,
                     out("v0") _, out("v1") _, out("v2") _,
                     out("v4") _, out("v5") _,
@@ -721,7 +702,7 @@ fn packed_packed_12x8_loop1(c: &mut Criterion) {
                     out("x4") _, out("x5") _, out("x6") _, out("x7") _,
                     out("x8") _, out("x9") _, out("x10") _, out("x11") _,
                     out("x12") _, out("x13") _, out("x14") _, out("x15") _,
-                    out("x19") _, out("x20") _, out("x21") _, out("x22") _,
+                    out("x20") _, out("x21") _, out("x22") _,
                     out("x23") _, out("x24") _, out("x25") _, out("x26") _,
                     out("v0") _, out("v1") _, out("v2") _,
                     out("v4") _, out("v5") _,
@@ -740,5 +721,43 @@ fn packed_packed_12x8_loop1(c: &mut Criterion) {
     bench_4_loop_1!(ldr_w_preload);
 }
 
-criterion_group!(benches, ld_64F32, packed_packed_12x8_loop1, packed_packed_8x8_loop1);
+fn packed_packed_16x4_loop1(c: &mut Criterion) {
+    let mut group = c.benchmark_group("packed_packed_16x4_loop1");
+    group.throughput(criterion::Throughput::Elements(16 * 4 * 4));
+
+    macro_rules! bench_4_loop_1 {
+        ($id:ident) => {
+            group.bench_function(stringify!($id), |b| {
+                b.iter(|| unsafe {
+                    let mut p = F32;
+                    let mut q = F32;
+                    r4!(asm!(include_str!(concat!("../arm64/arm64simd/arm64simd_mmm_f32_16x4/packed_packed_loop1/", stringify!($id), ".tmpli")),
+                    inout("x1") p, inout("x2") q,
+                    out("x4") _, out("x5") _, out("x6") _, out("x7") _,
+                    out("x8") _, out("x9") _, out("x10") _, out("x11") _,
+                    out("x12") _, out("x13") _, out("x14") _, out("x15") _,
+                    out("x20") _, out("x21") _, out("x22") _, out("x23") _,
+                    out("x24") _, out("x25") _, out("x26") _,
+                    out("v0") _, out("v1") _, out("v2") _, out("v4") _, out("v5") _,
+                    out("v16") _, out("v17") _, out("v18") _, out("v19") _,
+                    out("v20") _, out("v21") _, out("v22") _, out("v23") _,
+                    out("v24") _, out("v25") _, out("v26") _, out("v27") _,
+                    out("v28") _, out("v29") _, out("v30") _, out("v31") _,
+                    ));
+                })
+            });
+        }
+    }
+
+    bench_4_loop_1!(naive);
+    bench_4_loop_1!(cortex_a53);
+}
+
+criterion_group!(
+    benches,
+    ld_64F32,
+    packed_packed_12x8_loop1,
+    packed_packed_8x8_loop1,
+    packed_packed_16x4_loop1
+);
 criterion_main!(benches);
